//////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2005
//   Nicholas Kidd  <kidd@cs.wisc.edu>
//   Thomas Reps    <reps@cs.wisc.edu>
//   David Melski   <melski@grammatech.com>
//   Akash Lal      <akash@cs.wisc.edu>
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//   1. Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//
//   2. Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in
//      the documentation and/or other materials provided with the
//      distribution.
//
//   3. Neither the name of the University of Wisconsin, Madison,
//      nor GrammaTech, Inc., nor the names of the copyright holders
//      and contributors may be used to endorse or promote
//      products derived from this software without specific prior
//      written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//////////////////////////////////////////////////////////////////////////////
/*
 * $Id: keys.h,v 1.7 2005/05/12 16:48:07 kidd Exp $
 */
#ifndef KEYS_H_DEFINED
#define KEYS_H_DEFINED 1

#include <string>

using namespace wpds;
namespace wpds {
    class key_source;
}

extern wpds_key_t str2key( const char* );

extern wpds_key_t str2key( const std::string& s );

extern wpds_key_t new_str2key( const char * );

extern wpds_key_t int2key( int );

extern wpds_key_t create_key(key_source *);

extern wpds_size_t num_keys(void);

extern std::ostream& printkey(wpds_key_t, std::ostream&);

void showkey(wpds_key_t key, std::string &);

extern key_source *retrieve_item(wpds_key_t);

extern void traverse_dict(void (*f)(key_source *));
extern void unsafe_dict_clear();


#endif /* KEYS_H_DEFINED */
/*
 * $Log: keys.h,v $
 * Revision 1.7  2005/05/12 16:48:07  kidd
 * Added:
 *
 * wpds_key_t str2key( const std::string & s ) wrapper method so the user does
 * not need to use s.c_str() to create a string_src.
 *
 * Revision 1.6  2005/04/02 04:28:25  kidd
 * added a class key_source decl in keys.h for using it when not in wpds
 *
 * Revision 1.5  2005/03/09 22:18:57  kidd
 * Added the file COPYRIGHT and inserted this file at the beginning of each h and
 * cpp file.  Changed default CFLAG in Makefile to be "-ggdb -DDBGWPDS". This
 * initializes the integer "b_*_debug" flags to 1; however, turning compile time
 * warnings on or off is still successful as templates are instantiated at
 * compile time.
 *
 */
